//+------------------------------------------------------------------+
//|                                             Angle of average.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property  copyright "mladen"

#property  indicator_separate_window
#property  indicator_buffers 4
#property  indicator_color1  Green
#property  indicator_color2  Red
#property  indicator_color3  DimGray
#property  indicator_color4  Gray
#property  indicator_width1  2
#property  indicator_width2  2
#property  indicator_width4  2


//
//
//
//
//

extern int    MAPeriod         =  34;
extern int    MAType           =   1; // 4 -> LSMA
extern int    MAAppliedPrice   =   0;
extern double AngleLevel       =   8;
extern int    AngleBars        =   6;
extern bool   alertsOn         = false;
extern bool   alertsOnCurrent  = false;
extern bool   alertsMessage    = true;
extern bool   alertsSound      = false;
extern bool   alertsNotify     = false;
extern bool   alertsEmail      = false;

//
//
//
//
//

double Buffer1[];
double Buffer2[];
double Buffer3[];
double Buffer4[];
double state[];


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(5);
   SetIndexBuffer(0,Buffer1);
   SetIndexBuffer(1,Buffer2);
   SetIndexBuffer(2,Buffer3);
   SetIndexBuffer(3,Buffer4);
   SetIndexBuffer(4,state);
   SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexStyle(2,DRAW_HISTOGRAM);
   
   //
   //
   //
   //
   //
      
   string  MAsType;
   switch (MAType)
   {
      case 1:  MAsType="EMA";  break;
      case 2:  MAsType="SMMA"; break;
      case 3:  MAsType="LWMA"; break;
      case 4:  MAsType="LSMA"; break;
      default: MAsType="SMA";  break;
   }
   IndicatorShortName(MAsType+" angle ("+MAPeriod+","+DoubleToStr(AngleLevel,2)+","+AngleBars+")");
   IndicatorDigits(2);
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

#define Pi 3.141592653589793238462643

//
//
//
//
//

int start()
{
   int countedBars = IndicatorCounted();
   int limit, i;
 
   if(countedBars<0) return(-1);
   if(countedBars>0) countedBars--;
      limit = Bars-countedBars;

   //
   //
   //
   //
   //
   
   for(i=limit; i>=0; i--)
   {
      double range = iATR(NULL,0,AngleBars*20,i);
      double angle = 0.00;
      double change;
         if (MAType == 4)
               change = iLSMA(MAPeriod,MAAppliedPrice,i)-iLSMA(MAPeriod,MAAppliedPrice,i+AngleBars);
         else  change = iMA(NULL,0,MAPeriod,0,MAType,MAAppliedPrice,i)-iMA(NULL,0,MAPeriod,0,MAType,MAAppliedPrice,i+AngleBars);

         if (range != 0) angle = MathArctan(change/(range*AngleBars))*180.0/Pi;

      //
      //
      //
      //
      //
      
         Buffer1[i] = EMPTY_VALUE;
         Buffer2[i] = EMPTY_VALUE;
         Buffer3[i] = angle;
         Buffer4[i] = angle;
         state[i]   = 0;
            if (angle >  AngleLevel) { Buffer1[i] = angle; Buffer3[i] = EMPTY_VALUE; state[i] =  1;}
            if (angle < -AngleLevel) { Buffer2[i] = angle; Buffer3[i] = EMPTY_VALUE; state[i] = -1;}
   }
   manageAlerts();
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double iLSMA(int period,int appliedPrice,int shift)
{
   double lengthvar = (period + 1.0)/3.0;
   double sum       = 0.0;

   for(int i = period; i >= 1 ; i--)
      sum += (i-lengthvar)*iMA(NULL,0,1,0,MODE_SMA,appliedPrice,shift+period-i);
   return(sum*6.0/(period*(period+1.0)));
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;

      //
      //
      //
      //
      //
      
         if (state[whichBar] != state[whichBar+1])
         {
            if (state[whichBar] == 1)                            doAlert(whichBar," Angle crossed "+DoubleToStr( AngleLevel,2)+" up");
            if (state[whichBar] == 0 && state[whichBar+1] == 1)  doAlert(whichBar," Angle crossed "+DoubleToStr( AngleLevel,2)+" down");
            if (state[whichBar] == -1)                           doAlert(whichBar," Angle crossed "+DoubleToStr(-AngleLevel,2)+" down");
            if (state[whichBar] == 0 && state[whichBar+1] == -1) doAlert(whichBar," Angle crossed "+DoubleToStr(-AngleLevel,2)+" up");
         }         
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS),doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(StringConcatenate(Symbol(), Period() ," Angle of average : " +message));
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," - Angle of average"),message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
}